{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

Author:       francois.piette@overbyte.be, http://www.overbyte.be
Creation:     Jan 02, 2013
Description:  Demo application for PersistAs attribute
              Read blog article at http://francois-piette.blogspot.be/
Version:      1.00
History:


 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
unit AttributesDemoMain;

interface

uses
    Windows, Messages, SysUtils, Variants, Classes, Graphics,
    Controls, Forms, Dialogs, IniFiles, StdCtrls,
    TypInfo,
    AttrPersist;

type
    TConfig = class(TComponent)
    public
        [PersistAs('Config', 'Version', '1.0')]
        Version     : String;
        [PersistAs('Config', 'Description', 'No description')]
        Description : String;
        FTest        : Integer;
        // No attribute => not persistent
        Count       : Integer;
        [PersistAs('Config', 'Test', '0')]
        property Test : Integer read FTest write FTest;
    end;

    TAttributesDemoForm = class(TForm)
        WriteButton: TButton;
        DisplayMemo: TMemo;
        ReadButton: TButton;
        procedure WriteButtonClick(Sender: TObject);
        procedure ReadButtonClick(Sender: TObject);
    private
        FConfig      : TConfig;
        FIniFile     : TIniFile;
        FIniFileName : String;
        procedure AttrWriter(const Obj       : TObject;
                             const State     : TPersistState;
                             const ASection  : String;
                             const AKey      : String;
                             const AValue    : String);
        procedure AttrReader(const Obj       : TObject;
                             const State     : TPersistState;
                             const ASection  : String;
                             const AKey      : String;
                             const ADftValue : String;
                             out   AValue    : String);
    public
        constructor Create(AOwner : TComponent); override;
    end;

var
    AttributesDemoForm: TAttributesDemoForm;

implementation

{$R *.dfm}

{ TAttributesDemoForm }

constructor TAttributesDemoForm.Create(AOwner: TComponent);
begin
    inherited Create(Aowner);
    Displaymemo.Clear;
    FIniFileName        := ChangeFileExt(ParamStr(0), '.ini');
    FConfig             := TConfig.Create(Self);
    FConfig.Version     := '2.3';
    FConfig.Description := 'This is the demo';
    FConfig.Count       := 3;
end;

procedure TAttributesDemoForm.AttrWriter(
    const Obj      : TObject;
    const State    : TPersistState;
    const ASection : String;
    const AKey     : String;
    const AValue   : String);
begin
    DisplayMemo.Lines.Add(GetEnumName(TypeInfo(TPersistState),
                                      Ord(State)) + ', ' +
                          '"' + ASection + '", ' +
                          '"' + AKey     + '", ' +
                          '"' + AValue   + '"');
    case State of
    pstBeginUpdate : FIniFile := TIniFile.Create(FIniFileName);
    pstWriteField,
    pstWriteProp   : FIniFile.WriteString(ASection, AKey, AValue);
    pstEndUpdate   : FreeAndNil(FIniFile);
    end;
end;

procedure TAttributesDemoForm.AttrReader(
    const Obj       : TObject;
    const State     : TPersistState;
    const ASection  : String;
    const AKey      : String;
    const ADftValue : String;
    out   AValue    : String);
begin
    case State of
    pstBeginUpdate : FIniFile := TIniFile.Create(FIniFileName);
    pstReadField,
    pstReadProp    : AValue := FIniFile.ReadString(ASection, AKey, ADftValue);
    pstEndUpdate   : FreeAndNil(FIniFile);
    end;
    DisplayMemo.Lines.Add(GetEnumName(TypeInfo(TPersistState),
                                      Ord(State)) + ', ' +
                          '"' + ASection  + '", ' +
                          '"' + AKey      + '", ' +
                          '"' + AValue    + '"');
end;

procedure TAttributesDemoForm.WriteButtonClick(Sender: TObject);
begin
    DisplayMemo.Lines.Add('Write');
    TPersistHandler.Process(FConfig, AttrWriter);
end;

procedure TAttributesDemoForm.ReadButtonClick(Sender: TObject);
begin
    DisplayMemo.Lines.Add('Read');
    TPersistHandler.Process(FConfig, AttrReader);
end;

end.
